@@ -134,7 +134,7 @@ class ReactionNetwork(ABC):
     # if both empty the species are the same,
     return (not onlyOld and not onlyNew,onlyOld,onlyNew)
 
-  def compare(self,reactionNetwork,printresults=False,eqfunc=None,chgfunc=None):
+  def compare(self,reactionNetwork,printresults=False,eqfunc=None,chgfunc=None,log=True):
     '''
     Compares the network to the given reaction network.
     Currently simply prints out the results to stdout.
@@ -159,6 +159,9 @@ class ReactionNetwork(ABC):
       The function must take as an argument the outcome of the Reaction
       compare function.
 
+    log : boolean
+      Include log output (print statements) or not.
+
     Returns
     -------
     (tuple) :
@@ -170,7 +173,8 @@ class ReactionNetwork(ABC):
     '''
 
     starttime=timer()
-    print("INFO: compare ",self.name," to ",reactionNetwork.name," ...")
+
+    if log: print("INFO: compare ",self.name," to ",reactionNetwork.name," ...")
     # Copy the reaction list as we will change it
     reactionsNew=reactionNetwork.reactions.copy()
 
@@ -210,7 +214,7 @@ class ReactionNetwork(ABC):
 
     equal=len(changed)==0 and len(onlyold)==0 and len(onlynew)==0
 
-    print("INFO: time: ","{:4.2f}".format(timer()-starttime)+" s")
+    if log: print("INFO: time: ","{:4.2f}".format(timer()-starttime)+" s")
     if printresults:
       print(" ")
       print("FOUND "+str(len(changed))+" CHANGED REACTIONS: ")
@@ -372,6 +376,11 @@ class Reaction(object):
     * `products`
     * `coeffs`
     * `temps`
+
+    ..todo :
+      * deal with comparison of the network if rates only exist in one network
+      * the value of the rate coeffs are not compared/checked
+      * make the tolerances for comparison of the rate a parameter somehow
     '''
 
     equal=dict()
@@ -410,11 +419,12 @@ class Reaction(object):
         if not all(flat==flatNew):
           equal["temps"]=False
 
-    # TODO: if not loaded in both reactoins comparson does not make sense
+    # TODO: if not loaded in both reactions, the comparison does not make sense
+    # TOD
     # Maybe write an error.
     if self.rate!=None and reaction.rate!=None:
       # print(self.rate,reaction.rate,math.isclose(self.rate,reaction.rate,rel_tol=1e-20,abs_tol=0.0))
-      equal["rate"]=math.isclose(self.rate,reaction.rate,rel_tol=1e-6,abs_tol=0.0)
+      equal["rate"]=math.isclose(self.rate,reaction.rate,rel_tol=1e-4,abs_tol=0.0)
     else:
       equal["rate"]=True
 
@@ -547,6 +557,13 @@ class ReactionNetworkPout(ReactionNetwork):
     fr.close()
     print("Loaded ",len(self.reactions)," Reactions from ",self.filename)
 
+  def _equalreaction(self,eq):
+    '''
+    Are the reactions equal?
+    '''
+    # here we also have the type
+    return super()._equalreaction(eq) and eq["type"]==True
+
   def load_rates(self,filename="rate_coeffs_1NZZ.log"):
     '''
     Load the real rates for a given set of physical conditions.
