@@ -5,6 +5,7 @@ from __future__ import unicode_literals
 import difflib
 import glob
 from inspect import ismethod
+import inspect
 import os
 
 from astropy.io.fits.diff import FITSDiff
@@ -19,9 +20,8 @@ class CompareAbs(object):
   A subclass of this class needs to implement the necessary compare routine(s).
   (example see :class:`Compare`)
 
-
-  TODO: try once to implement def __eq__(self, other):  function in the data
-  containers and use that for the comparison.
+  .. todo::
+    * make tolerances configurable via config file (see punit checkModels).
 
   """
 
@@ -321,13 +321,43 @@ class Compare(CompareAbs):
   """
 
   def __init__(self,model,modelref):
+    """
+    Parameters
+    ----------
+    model : :class:`~prodimopy.read_mc.Data_mc`
+      The model one wants to compare.
+
+    modelref : :class:`~prodimopy.read_mc.Data_mc`
+      The reference model to compare with.
+
+    Attributes
+    ----------
+
+    """
 
     if model is None or modelref is None:
       raise TypeError("Either model or modelref is None, no comparison possible.")
 
     self.m=model
+    """ :class:`~prodimopy.read.Data_ProDiMo` :
+    The model one wants to compare.
+    """
     self.mref=modelref
+    """ :class:`~prodimopy.read.Data_ProDiMo` :
+    The reference model agains which model is compared..
+    """
+    self.tolerances={}
+    """ dict :
+    A dictionary of tolerances. The naming convention has to follow the naming
+    of the compare Function. e.g. If one wants to provide tolerances for
+    `compareAbundances` the key names are `Abundances.rtol` and `Abundances.atol`.
+
+    FIXME: This is still experimental and not implemented for all compare routines.
+
+    """
+
     # the default allowed difference
+
 #     self.d=1.e-3
 #     self.dcdnmol=0.5  # the allowed differences for the column densities (radial and vertical)
 #                       # chemistry is difficult and uncertain :) FIXME: this has to become better, there
@@ -352,17 +382,48 @@ class Compare(CompareAbs):
     self.m._log=False
     self.mref._log=False
 
+  def getTols(self,atol=None,rtol=None,nlimfrac=None):
+    '''
+    Get the tolerance values from the `self.tolerances` dictionary
+
+    Returns
+    -------
+
+    dict: containing {"atol": value, "rtol": value } if set. Can also be empty
+
+    '''
+
+    # This gives me the name of the caller function. Check if it is a compare
+    # function
+    callername=inspect.currentframe().f_back.f_code.co_name
+    if not callername.startswith("compare"):
+      raise ValueError("getTols can only be called from a compare function!")
+
+    # strip the compare
+    valname=callername.replace("compare","")
+
+    # The dictonary that contains the return values, if any
+    retdict={}
+
+    for key,defval in zip(["atol","rtol","nlimfrac"],[atol,rtol,nlimfrac]):
+      if valname+"."+key in self.tolerances:
+        retdict[key]=self.tolerances[valname+"."+key]
+      elif defval is not None:
+        retdict[key]=defval
+
+    return retdict
+
   def compareX(self):
     '''
     Compares the x grid points.
     '''
-    return self.diffArray(self.m.x,self.mref.x,rtol=1.e-5)
+    return self.diffArray(self.m.x,self.mref.x,**self.getTols(rtol=1.e-5))
 
   def compareZ(self):
     '''
     Compares the z grid points.
     '''
-    ret=self.diffArray(self.m.z,self.mref.z,rtol=1.e-5)
+    ret=self.diffArray(self.m.z,self.mref.z,**self.getTols(rtol=1.e-5))
     return ret
 
   def compareLineFluxes(self):
@@ -378,7 +439,7 @@ class Compare(CompareAbs):
 
     mFluxes=np.array([x.flux for x in self.m.lines])
     mrefFluxes=np.array([x.flux for x in self.mref.lines])
-    return self.diffArray(mFluxes,mrefFluxes,rtol=0.05)
+    return self.diffArray(mFluxes,mrefFluxes,**self.getTols(rtol=0.05))
 
   def compareLineEstimates(self):
     '''
@@ -398,7 +459,7 @@ class Compare(CompareAbs):
 
     mask=np.logical_and(mFluxes<1.e-24,mrefFluxes<1.e-24)
 
-    return self.diffArray(mFluxes,mrefFluxes,rtol=3.e-1,nlimfrac=0.01,mask=mask)
+    return self.diffArray(mFluxes,mrefFluxes,mask=mask,**self.getTols(rtol=3.e-1,nlimfrac=0.01))
 
   def compareSED(self):
     """
@@ -407,7 +468,7 @@ class Compare(CompareAbs):
     if self.m.sed is None and self.mref.sed is None: return True,None
     if self.m.sed is not None and self.mref.sed is None: return False,None
     if self.m.sed is None and self.mref.sed is not None: return False,None
-    return self.diffArray(self.m.sed.fnuErg,self.mref.sed.fnuErg,rtol=1.e-3)
+    return self.diffArray(self.m.sed.fnuErg,self.mref.sed.fnuErg,**self.getTols(rtol=1.e-3))
 
   def compareContinuumImages(self):
     """
@@ -447,7 +508,7 @@ class Compare(CompareAbs):
     # for i in specidxMref:
     #   self.mref.cdnmol[self.mref.cdnmol[:,0,i]<self.lcdnmol,0,i]=self.lcdnmol
 
-    return self.diffArray(self.m.cdnmol[:,0,:],self.mref.cdnmol[:,0,:],1.e-1,1.0,0.1)
+    return self.diffArray(self.m.cdnmol[:,0,:],self.mref.cdnmol[:,0,:],**self.getTols(rtol=1.e-1,nlimfrac=0.1))
     # ok,d,b=self.diffArray(self.m.cdnmol[:,0,specidxM],self.mref.cdnmol[:,0,specidxMref],1.e-1,1.0,0.1)
 
     # if false check if it is only a certain fraction of the columns, it than can
@@ -482,7 +543,7 @@ class Compare(CompareAbs):
     # for i in specidxMref:
     #   self.mref.rcdnmol[-1,self.mref.rcdnmol[-1,:,i]<self.lcdnmol,i]=self.lcdnmol
 
-    return self.diffArray(self.m.rcdnmol[-1,:,:],self.mref.rcdnmol[-1,:,:],1.e-1,1.0,0.1)
+    return self.diffArray(self.m.rcdnmol[-1,:,:],self.mref.rcdnmol[-1,:,:],**self.getTols(rtol=1.e-1,nlimfrac=0.1))
     # ok,d,b=self.diffArray(self.m.rcdnmol[-1,:,specidxM],self.mref.rcdnmol[-1,:,specidxMref],1.e-1,1.0,0.1)
 
     # if false check if it is only a certain fraction of the columns, it than can
@@ -503,7 +564,7 @@ class Compare(CompareAbs):
     '''
     Compares the solved_chem field. The flag for the quality of the chemical solution.
     '''
-    return self.diffArray(self.m.solved_chem,self.mref.solved_chem,rtol=1.e-10,nlimfrac=0.1)
+    return self.diffArray(self.m.solved_chem,self.mref.solved_chem,**self.getTols(rtol=1.e-10,nlimfrac=0.1))
 
   def compareAbundances(self):
     """
@@ -541,7 +602,7 @@ class Compare(CompareAbs):
     for i in range(len(specs)):
       mask[:,:,i]=np.logical_or(mask[:,:,i],masknotsolved)
 
-    return self.diffArray(am,amref,rtol=1.e-3,nlimfrac=0.05,mask=mask)
+    return self.diffArray(am,amref,mask=mask,**self.getTols(rtol=1.e-3,nlimfrac=0.05))
 
   def compareElements(self):
     '''
@@ -561,6 +622,22 @@ class Compare(CompareAbs):
     return self.diffArray(np.array(list(self.m.elements.abun12.values())),
                           np.array(list(self.mref.elements.abun12.values())),rtol=1.e-5)
 
+  def compareSpecies(self):
+    '''
+    Compares the numbers from the Species.out file.
+    '''
+
+    ok,val,b=self.diffArray(np.array(list(self.m.species.mass.values())),
+                          np.array(list(self.mref.species.mass.values())),rtol=1.e-5)
+    if ok==False: return ok,val,b
+
+    ok,val,b=self.diffArray(np.array(list(self.m.species.charge.values())),
+                          np.array(list(self.mref.species.charge.values())),rtol=1.e-5)
+    if ok==False: return ok,val,b
+
+    return self.diffArray(np.array(list(self.m.species.chemPot.values())),
+                          np.array(list(self.mref.species.chemPot.values())),rtol=1.e-5)
+
   def compareDustOpacEnv(self):
     '''
     Compares the dust opacities for an envelope model (optional output).
@@ -569,13 +646,14 @@ class Compare(CompareAbs):
     if self.m.env_dust is not None and self.mref.env_dust is None: return False,None
     if self.m.env_dust is None and self.mref.env_dust is not None: return False,None
 
-    return self.diffArray(self.m.env_dust.kext,self.mref.env_dust.kext,rtol=1.e-3)
+    return self.diffArray(self.m.env_dust.kext,self.mref.env_dust.kext,**self.getTols(rtol=1.e-3))
 
   def compareDustOpac(self):
     '''
     Compares the dust opacities.
     '''
-    return self.diffArray(self.m.dust.kext,self.mref.dust.kext,rtol=1.e-3)
+    # self.tolerances["DustOpac.rtol"]=0.06
+    return self.diffArray(self.m.dust.kext,self.mref.dust.kext,**self.getTols(rtol=1.e-3))
 
   def compareStarSpec(self):
     '''
@@ -615,21 +693,27 @@ class Compare(CompareAbs):
 
   def compareTg(self):
     '''
-    checks the gas Temperature
+    checks the gas temperature
     '''
-    return self.diffArray(self.m.tg,self.mref.tg,rtol=2.e-2,nlimfrac=0.01)
+    return self.diffArray(self.m.tg,self.mref.tg,**self.getTols(rtol=2.e-2,nlimfrac=0.01))
 
   def compareTd(self):
     '''
-    checks the dust Temperature
+    checks the dust temperature
     '''
-    return self.diffArray(self.m.td,self.mref.td,rtol=2.e-3,nlimfrac=0.01)
+    return self.diffArray(self.m.td,self.mref.td,**self.getTols(rtol=2.e-3,nlimfrac=0.01))
+
+  def compareRadFields(self):
+    '''
+    checks the radiation fields
+    '''
+    return self.diffArray(self.m.radFields,self.mref.radFields,**self.getTols(rtol=2.e-2,nlimfrac=0.01))
 
   def compareZetaCR(self):
     '''
     checks the cosmic ray ionisation rate
     '''
-    return self.diffArray(self.m.zetaCR,self.mref.zetaCR,rtol=1.e-5)
+    return self.diffArray(self.m.zetaCR,self.mref.zetaCR,**self.getTols(rtol=1.e-5))
 
   def compareZetaX(self):
     '''
@@ -638,7 +722,7 @@ class Compare(CompareAbs):
 
     mask=np.logical_and(self.m.zetaX<1.e-25,self.mref.zetaX<1.e-25)
 
-    return self.diffArray(self.m.zetaX,self.mref.zetaX,rtol=1.e-1,mask=mask,nlimfrac=0.05)
+    return self.diffArray(self.m.zetaX,self.mref.zetaX,mask=mask,**self.getTols(rtol=1.e-1,nlimfrac=0.05))
 
   def compareHX(self):
     '''
@@ -648,19 +732,15 @@ class Compare(CompareAbs):
 #    self.m.Hx[self.m.zetaX < self.lZetaX]=self.lZetaX
 #    self.mref.Hx[self.mref.zetaX < self.lZetaX]=self.lZetaX
 
-    return self.diffArray(self.m.Hx,self.mref.Hx,rtol=1.e-1,nlimfrac=0.05)
+    return self.diffArray(self.m.Hx,self.mref.Hx,**self.getTols(rtol=1.e-1,nlimfrac=0.05))
 
-  def compareFileReactionsOut(self):
+  def compareChemnet(self):
     '''
-    Makes a file comparison with Reactions.out.
+    Compare the chemical network. Uses the compare routine from `:class:~prodimopy.chemistry.network.ReactionNetwork
     '''
-    return self.diffFile("Reactions.out")
-
-  def compareFileSpeciesOut(self):
-    '''
-    Makes a file comparison with Species.out
-    '''
-    return self.diffFile("Species.out")
+    # only tell if it is equal or  not, i.e. ingore all the other output
+    ret=self.m.chemnet.compare(self.mref.chemnet,printresults=False,log=False)
+    return (ret[0],None,None)
 
   def compareFileCheckNetworkLog(self):
     '''
