@@ -1541,9 +1541,25 @@ class Plot(object):
                     norm=None,styles=None,colors=None,markers=None,linewidths=None,
                     ax=None,**kwargs):
     '''
-    Plot vertical radial abundance in the midplane
-    (can be more than one)
-    Same as abunvert but radially more usefull for envelopes
+    Plots species abundances as function of radius in the midplane (z=0)
+    Similar to abunvert but radially is more usefull for e.g. envelope structures.
+
+    Parameters
+    ----------
+    model : :class:`prodimopy.read.Data_ProDiMo`
+      the model data
+
+    species : array_like(str,ndim=1) :
+      List of species names to plot.
+
+    useNH : boolean
+      plot the abundances as function of radial column densities
+      Default: `False`
+
+    norm : float
+      normalize the y values by the given number (i.e. y=y/norm)
+      Default: `None` (i.e. no normalisation)
+      
     '''
 
     print("PLOT: plot_abunrad ...")
@@ -1618,8 +1634,21 @@ class Plot(object):
   def plot_abun_midp(self,model,species,norm=None,styles=None,colors=None,ax=None,**kwargs):
     '''
     Plots the abundances in the midplane for the given species (can be more than one)
-    norm .. normalisation constant (additional to Nhto e.g. the total carbon abundance)
+
+    Parameters
+    ----------
+    model : :class:`prodimopy.read.Data_ProDiMo`
+      the model data
+
+    species : array_like(str,ndim=1) :
+      List of species names to plot.
+
+    norm : float
+      normalize the y values by the given number (i.e. y=y/norm)
+      Default: `None` (i.e. no normalisation)
+      
     '''
+
     print("PLOT: plot_abun_midp ...")
     fig,ax=self._initfig(ax,**kwargs)
 
