@@ -268,8 +268,11 @@ class PlotCasasim(object):
         else:
           velidx=cvel_idx+(iax-naxesh)*step
 
-        im=ax.imshow(cube.data[velidx,:,:]*scalefac,cmap=cmap,norm=norm,
+        if norm is None:
+          im=ax.imshow(cube.data[velidx,:,:]*scalefac,cmap=cmap,norm=norm,
                        vmin=vmin,vmax=vmax,origin="lower")
+        else:
+          im=ax.imshow(cube.data[velidx,:,:]*scalefac,cmap=cmap,norm=norm,origin="lower")
 
         if zoomto is not None:
           if type(zoomto) is list:
